'' ILI9325 driver using pins P0-P11 for the display and P16-P23 for the touchscreen
'' James Moxham 2011

CON


  _clkmode      = xtal1 + pll16x                        ' use crystal x 16
  _xinfreq      = 5_000_000

  _SD_DO = 12
  _SD_CLK = 13
  _SD_DI = 14
  _SD_CS = 15
  _SD_WP = -1                                           ' -1 ifnot installed.
  _SD_CD = -1                                           ' -1 ifnot installed.

  _RTC_DAT = -1                                         ' -1 ifnot installed. For kye's sd driver
  _RTC_CLK = -1                                         ' -1 ifnot installed.

  _TV_Enabled = true                                    ' TV display on
  _TV_Pin = 16                                          ' group of 3 pins starting at pin 16 = 16,17,18

  Touch_Clock = 21              ' 23 old board                          ' Touch screen pins (shared with VGA and one, penirq, shared with TV)
  Touch_ChipSelect = 23         ' 22 old board
  Touch_DataIn = 22             ' 21 old board
  'Touch_Busy = 20              ' 20 old board
  Touch_DataOut = 20            ' 19 old board
  'Touch_Penirq = 18             ' 18 old board
  
  ' Memory map - aligned on 256 byte blocks for faster block reads
  ' Display from 0 - 153599
  ' Icons from 153600 to 169983 ' store background behind the icon
  ' A text buffer from 169984 to 202751
  ' The current font from 202752 up

  RamText   = 169984
  FontTable = 202752 ' all fonttables and anything that uses the A command must start on 0,256,512,768 etc
                     ' .ifn font files have been rounded up so always fit in blocks of 256 (check the file size)
                     ' so read in the file size and then can put fonts one after the other 
                     ' on the other hand these files are big and may cause memory management problems
                     ' so may be better to only have one font at a time
                     ' note that there is kerning data as well with combinations of letters and the amount to shuffle
OBJ
 delay:         "timing"                                ' thanks to parallax
 fat:           "SD3.01_FATEngine.spin"                 ' thanks to Kye
 text:          "tv_text"                               ' thanks to Chip
 str:           "ASCII0_STREngine.spin"                 ' thanks to Kye 
 spi:           "SPI_ASM"                               ' thanks to Beau
 ILI9325:       "ILI9325_Dracblade"                     ' J Moxham

VAR
   byte LineOfText1[20] ' general purpose string buffer
   byte LineOfText2[20] ' general purpose string buffer
   byte LineOfText3[20] ' general purpose string buffer
   byte FontHeight                ' size of the current loaded font (pixels = fontsize *.75)
   word BackFontColor           ' the background color in RRRRRGGG GGGBBBBB format
   word curx                    ' current cursor x position
   word cury                    ' current cursor y position   
  
PUB Main
    text.start(_TV_Pin)                                 ' start the TV driver
    text.clearscreen                                    ' clear the screen
    ILI9325.start_ram                                   ' start the external ram driver in a cog
    ILI9325.start_ILI9325                               ' start the ILI9325 display
    PrintStringCR(string("Testing for SD card"))       ' helpful message if card is out, better than just a blank screen  
    'Touchscreen                                        ' test the touchscreen
    ifnot( fat.FATEngineStart(_SD_DO, _SD_CLK, _SD_DI, _SD_CS, _SD_WP, _SD_CD, _RTC_DAT, _RTC_CLK, -1))   
      reboot
    fat.mountPartition(0)                               ' mount the sd card
    TouchStart
    ' Boot message
    Clearscreen
    str.copy(string("JDLCD.ifn"),@lineoftext1)          ' new faster font file - one file, designed to be read as fast as possible
    ILIloadfont                              
    ILISetcursor(0,0)                                   ' set cursor to this location
    ClearFontBackground(0,0,239,FontHeight)                 ' clear an area the same color as the font background       
    ILIprintstring(string(">Booting"))

    'ILIdebugString(string("Debug"))                     ' debug string and hex value
    'ILIdebugHex($03A4EF27)

    DrawBackgroundAndIcons 
    Touchscreen

PUB DrawBackgroundAndIcons
    str.copy(string("beach.raw"),@lineoftext1)
    FullScreenRaw                                       ' load picture 240 wide, 320 high
    DisplayIcons

PUB PhotoAlbum | picturenumber
   picturenumber := 1
   repeat until picturenumber > 4
     if picturenumber == 1
        str.copy(string("cat.raw"),@lineoftext1)    
     if picturenumber == 2
        str.copy(string("kluger.raw"),@lineoftext1)
     if picturenumber == 3
        str.copy(string("elle.raw"),@lineoftext1)    
     if picturenumber == 4
        str.copy(string("balloon.raw"),@lineoftext1)
     FullScreenRaw                                      ' draw picture name loaded into lineoftext1
     result := Touchswish
     if result > 0
       picturenumber --
     else
       picturenumber ++

PUB ILILoadFont | filesize,i                ' read a .ifn file premade in angelcode bmfont and then the vb.net program
    ' pass file name in lineoftext2
    ' the variable "i" must start on 0,256,512,768 etc for the block move to work
    i := fonttable                                      ' store at the constant 'fonttable' location
    fat.openfile(@lineoftext1,"R")                      ' open an image. Size is in the first long and has been rounded up to the next 256 byte block
    filesize := fat.filesize                            ' read the file size. This file has been rounded up to the next 256 byte block
    repeat (filesize >> 8)                              ' read in this number of 256 byte blocks
      fat.readdata(@sdbuffer,256)                       ' get 256 bytes from sd card  
      ILI9325.docmd("A",@sdbuffer,i,256)      ' A must start on 0,256,512,768 etc move data from hub to external ram 256 bytes at a time    
      i += 256                             ' add 256
    fat.closefile
    FontHeight := ILI9325.readrambyte(fonttable+251)    ' extract font size
    ' extract the background color so can draw the background behind the text
    repeat i from 0 to 2
      sdbuffer[i] := ILI9325.readrambyte(fonttable + 35+i)   ' read the 3 RGB bytes
    ILI9325.docmd("E",@sdbuffer,@buffer2,1)             ' convert to 2 byte .ili format, len is number of pixels        
    BackFontColor := word[@buffer2][0]
    result := filesize

      
PUB ILIChar(ascii,x,y) | jump,size,width,height,ramaddress,xoffset,yoffset,xadvance   ' read out a character from a .ifn file stored in ram
  ' pass Fonttable = global
    jump := ILI9325.ReadRamLong((ascii << 2) + Fonttable + 256)' jump location = ascii *4 plus fonttable + 256 
    jump += fonttable                                    ' add the fonttable location
    size := ILI9325.ReadRamByte(jump) | (ILI9325.ReadRamByte(jump+1) << 8) ' width x height * 2 precalculated
    width := ILI9325.ReadRamByte(jump+2)                ' width
    height := ILI9325.ReadRamByte(jump+3)               ' height
    xoffset := ILI9325.ReadRamByte(jump+4)              ' xoffset
    yoffset := ILI9325.ReadRamByte(jump+5)              ' yoffset
    xadvance := ILI9325.ReadRamByte(jump+6)             ' move x by this amount for next char
    if (x+width-1+xoffset) < 239                          ' don't try to display if off the edge
      ILI9325.draw(x+xoffset,y+yoffset,x+width-1+xoffset,y+height-1+yoffset)      ' draw on screen
      ILI9325.docmd("C",0,jump+16,size)              ' move bytes from ram out to the display
    return x + xadvance


PUB ILIPrintstring(stringptr)                 'print at curx,cury
'' Print a zero-terminated string
  repeat strsize(stringptr)
    curx := ILIChar(byte[stringptr++],curx,cury)                           


PUB ILISetCursor(x,y)            
    curx := x
    cury := y

PUB ILIPrintstringcrlf(stringptr) ' print string and crlf
    ILIprintstring(stringptr)
    ILIcrlf

PUB ILIdebughex(number) ' print a debug value on the screen in hex
    str.copy(string("Arial40.ifn"),@lineoftext1)          ' new faster font file - one file, designed to be read as fast as possible
    ILILoadFont
    ILISetcursor(0,279)                              ' set cursor to this location
    ClearFontBackground(0,279,239,279+FontHeight)                 ' clear an area the same color as the font background       
    ILIPrintstring(string("0x"))
    str.copy(str.integertohexadecimal(number,8),@lineoftext1)
    ILIPrintstring(@lineoftext1)

PUB ILIdebugString(stringptr) ' print a debug string on the screen
    str.copy(string("Arial40.ifn"),@lineoftext1)          ' new faster font file - one file, designed to be read as fast as possible
    ILILoadFont
    ILISetcursor(0,239)                              ' set cursor to this location
    ClearFontBackground(0,239,239,239+FontHeight)                 ' clear an area the same color as the font background       
    ILIprintstring(stringptr)

'PUB ILIDecimal(number)
'      str.str(@lineoftext1,number) ' decimal to string with no leading zeros
'      ILIPrintstring(@lineoftext1)

      'str.copy(str.integertodecimal(number,10),@lineoftext1) ' or add lots of leading zeros. Your choice
      'ILIstr(@lineoftext1) 

PUB ILIcrlf
    curx := 0
    cury += FontHeight
    ClearFontBackground(0,cury,239,cury + FontHeight) ' clear background line to the background color ready to draw          
                     
PUB ReadBackground | ramaddress             ' block move data from sd card to memory, much faster than one byte at a time
    'PrintStringCR(string("Block load"))                 ' message on the TV screen
    fat.openfile(string("icons320.ili"),"R")            ' open an image (.ili extension)
    ramaddress := 0
    repeat 600
      fat.readdata(@sdbuffer,256)                       ' get 256 bytes from sd card
      ILI9325.docmd("A",@sdbuffer,ramaddress,256)       ' A must start on 0,256,512,768 etc move data from hub to external ram 256 bytes at a time    
      ramaddress := ramaddress + 256                    ' add 256
    fat.closefile   
    'PrintStringCR(string("Closing file"))
    RamToDisplay


PUB FullScreenRaw | ramaddress                         ' convert 3 byte .raw file to 2 byte .ili file
    fat.openfile(@lineoftext1,"R")                ' open an image
    ramaddress := 0
    repeat 600
      fat.readdata(@sdbuffer,384)                       ' 128 pixels, 3 bytes per pixel
      ILI9325.docmd("E",@sdbuffer,@buffer2,128)         ' convert to 2 byte .ili format, len is number of pixels
      ILI9325.docmd("A",@buffer2,ramaddress,256)        ' A must start on 0,256,512,768 etc move bytes to external ram
      ramaddress := ramaddress + 256                    ' add 256
    fat.closefile
    RamToDisplay                              ' move bytes from ram out to display

PUB DrawKeyboard | ramaddress
    ' reload the font as gets overwritten by icons
    str.copy(string("mssans12.ifn"),@lineoftext1)          
    ILILoadfont
    fat.openfile(string("keyboard.raw"),"R")                ' open the keyboard picture   
    ramaddress := 0
    repeat 360
      fat.readdata(@sdbuffer,384)                       ' 128 pixels, 3 bytes per pixel
      ILI9325.docmd("E",@sdbuffer,@buffer2,128)         ' convert to 2 byte .ili format, len is number of pixels
      ILI9325.docmd("A",@buffer2,ramaddress,256)        ' A must start on 0,256,512,768 etc move 256 bytes to external ram (must be 256)
      ramaddress += 256                                 ' add 256
    fat.closefile
    ILI9325.draw(0,128,239,319)                           ' 240x192 for the keyboard
    ILI9325.docmd("C",0,0,92160)                       ' move bytes from ram out to the display
    
PUB Touchkeyboard  | yval, xval,address,ascii,index,capslock, backspace
    ' from the top of the keyboard picture - add 128 for screen pixel 0-319
    ' y=33-61 is the numbers ( 161 - 189 which is 50% to 59%) 
    ' y = 64-93 is qwerty    (192 to 221 which is 60 to 69%)
    ' y = 96-124 is asdf      (224 to 252 which is 70-79%)
    ' y= 128-157 is zxcv      (256 to 285 which is 80 to 89%)
    ' y=160-189 is space
    capslock := 0 ' default to off
    backspace := 0 'default to off
    ILISetcursor(10,137)                              ' set cursor to this location    
    repeat
      yval := TouchValue >> 8                               ' decode yval 0-100%               
      xval := TouchValue & 255                              ' decode xval 0-100%
      if (xval <> 255)  and (yval <> 255)               ' a valid value

        if (yval => 50) and (yval =< 59)                ' number row 0-100%
          address := xval / 10                              ' 10 values in row = 0 to 9
          'ILIdecimal(address)                                     ' test debugging on the ILI screen.

        if (yval => 60) and (yval =< 69)
          address := 10 + (xval / 10)                   ' address = 10 to 19    QWERTYUIOP

        if (yval => 70) and (yval =< 79)
          address := 20 + ((xval - 5) / 10)             ' address = 20-28 ASDFGHJKL shift left slightly as this row is offset half a key

        if (yval => 80) and (yval =< 89)                ' third row
          if (xval < 15)
            !capslock                                   ' swap 0,FFFFFFFF capslock
            address := 255                              ' so no character printed
            'printstringcr(string("Caps lock"))
          if (xval =>15) and (xval =< 85)
            address := 29 + ((xval - 15) / 10)          ' ZXCVBNM
          if xval > 85
            backspace := 255                             ' backspace - need to be storing in a string and delete the last character and reprint the string                             

        if yval => 90                                   ' fourth row
           if xval <25
           ' not sure what to do with this 123 button, maybe all the !@#$%^ characters above the numbers
           if (xval => 25) and (xval =< 75)
             address := 36                              ' space  button
           if xval > 75                                 ' enter button
           ' exit here and return a string
           return
           
        if address < 255                                ' not a backspace or an enter or a capslock
          ascii := lookupz(address: "1","2","3","4","5","6","7","8","9","0","Q","W","E","R","T","Y","U","I","O","P","A","S","D","F","G","H","J","K","L","Z","X","C","V","B","N","M"," ")
          if (capslock <> 0) and (ascii => 65) and (ascii =< 90) 
            ascii += 32                                 ' lower case
          curx := ILIChar(ascii,curx,cury)
             
      delay.pause1ms(250)' only display once so delay half a second to "debounce"   
            
{    
PUB CreateIcon | iconaddress,maskaddress,i,j
  ' this creates a .icn file which is 4 bytes per pixel, red, green, blue and alpha. Do once when setting up icons
  ' so one long per pixel. All values are 0-255
  ' takes 8 seconds per icon so don't do this every load
  ' to create an icon using paint shop pro or similar
  ' in the catalina IDE this has been automated to a one click process for any .png file
  '1) Open a .png icon file which has a transparancy layer
  '2) Copy and paste the picture to a new picture and save as a .raw RRRRRRRRGGGGGGGGBBBBBBBB file
  '3) View the mask (in PSP there is a Mask on the top menu bar, then Edit).
  '4) Masks/Save the file, and rename the extension as .bmp.
  '5) Close down all open files in PSP, then File/Open and open the .bmp file above. This should be 256 color grayscale, ie 1 byte per pixel.
  '6) Save that file as a .raw file, eg mask.raw
  '7) Copy the mask.raw file to the sd card This .raw file has 1 byte per pixel (note different to the picture which is 3 bytes per pixel)    

  ' in a family of icons the mask.raw file is going to be the same for all of them
  ' so on the sd card one might have two files - clock.raw and mask.raw
  ' this function combines these together into an .icn file. It is in spin which is slower but this is only ever run once
  PrintStringCR(string("myfile.raw and mask.raw to myfile.icn"))   
  fat.openfile(string("clock.raw"),"R")                 ' open an icon file which is 59x60 bytes x3 = 10620
  iconaddress := 0 
  repeat 10620                                          ' 
    i := fat.readbyte
    ILI9325.docmd("W",@i,iconaddress,1)                  ' write n to ramaddress
    iconaddress +=1                                     ' increment the counter
  fat.closefile
  fat.openfile(string("mask.raw"),"R")                 ' open an icon file which is 59x60 bytes x3 = 10620
  maskaddress := 16384
  repeat 3540                                           ' one byte per pixel in the mask
    i := fat.readbyte
    ILI9325.docmd("W",@i,maskaddress,1)                  ' write n to ramaddress
    maskaddress += 1                                    ' increment the counter
  fat.closefile
  iconaddress := 0                                      ' reset counters
  maskaddress := 16384
                                                        ' see kyedos for using "stringpointer" instead of text 
  result := \fat.newFile(string("clock.icn"))           ' All aborts traps must have "variable := \function" setup.
  if(fat.partitionError == fat#Entry_Already_Exist)     ' file already exists
     result := \fat.deleteEntry(string("clock.icn"))    ' delete the old file
     result := \fat.newfile(string("clock.icn"))        ' recreate the file
  fat.openfile(string("clock.icn"),"W")
  repeat 3540
    j := 0                                              ' clear j
    ILI9325.docmd("R",@i,iconaddress,1)                 ' read from ram into i = the red byte
    iconaddress += 1
    j := j | i                                          ' put the red byte into lsb place
    ILI9325.docmd("R",@i,iconaddress,1)                 ' read from ram
    iconaddress += 1
    j := j | (i << 8)                                   ' put the green byte into place
    ILI9325.docmd("R",@i,iconaddress,1)                 ' read from ram 
    iconaddress += 1
    j := j | (i << 16)                                  ' put the blue byte into place
    ILI9325.docmd("R",@i,maskaddress,1)                 ' read from ram into i the alpha byte
    maskaddress += 1
    j := j | (i << 24)                                  ' put the alpha byte into msb place - hex editor = RGBA
    fat.writelong(j)                                    ' store to SD card file
  fat.closefile
  PrintStringCR(string("clock.icn created"))                                                                           
}
PUB RamToDisplay ' dumps preloaded ram to display starting address 0
    'Clearscreen                                        ' leave out clearscreen
    ILI9325.draw(0,0,239,319)                           ' set up the area of the screen to draw in
    ILI9325.docmd("C",0,0,153600)                       ' move bytes from ram out to the display

PUB Movie                                              ' maybe need to speed up the draw routine as this is in spin
    clearscreen
    fat.openfile(string("taz.ilm"),"R")             ' also can skip alt frames, and maybe make the sd buffer bigger
    repeat 338 ' number of frames
      ILI9325.draw(60,115,179,204)                           ' this can be sped up as this is in spin
      repeat 45                                         ' 120*90*2 / 480
        fat.readdata(@sdbuffer,480)                     ' read data into hub
        ILI9325.docmd("D",@sdbuffer,0,480)              ' send data out to the display (could possibly do some in parallel, though the ILI display may be faster than the sd card)
    fat.closefile
PUB Clearscreen | i
    i := 0
    repeat 512
      sdbuffer[i] :=0                         ' array all to zero = black, 255 = white, other colors need to work out the RRRRRGGG GGGBBBBB values
      i := i + 1
    ILI9325.draw(0,0,239,319)                           ' set up the area of the screen to draw in   
    repeat 300
      ILI9325.docmd("D",@sdbuffer,0,512)                       ' move bytes from ram out to the display

PUB ClearFontBackground(x1,y1,x2,y2) | i ' clears an area of the screen to the current font background color
    ' maximum width is 256 pixels so do in two parts if in landscape mode
    repeat i from 0 to 510 step 2                         ' move the background font colour to the buffer
      sdbuffer[i] := BackFontColor & 255
      sdbuffer[i+1] := BackFontColor >> 8
    ILI9325.draw(x1,y1,x2,y2)                           ' set up the area of the screen to draw in
    i := (x2 - x1 + 1) << 1                             ' number of columns * 2 = number of bytes to send
    repeat (y2 - y1 +1)                                 ' do this many rows
      ILI9325.docmd("D",@sdbuffer,0,i)                  ' 2 = 2 bytes which = one pixel 
 

PUB TransparentIcon(x,y) | sizex,sizey,backgroundaddress,ramaddress ' draw icon name in LineOfText1
    x := x * 60 ' pass x as 0,1,2,3 from touchscreen convert to a pixel coordinate
    y := y * 60 ' pass y as 0,1,2,3,4 from touchscreen
    sizex := 59
    sizey := 60
    ILI9325.draw(x,y,x+sizex-1,y+sizey-1)                           ' set up the area of the screen to draw in
     ' background is stored in memory 0 to 153599
     ' background behind the icon temp store =64*64*4 = 16384 bytes 153600 to 169984 so contiguous bytes
    backgroundaddress := (((y*240)+x) << 1)              ' calculate the correct location in background ram
    ramaddress := 153600
    repeat 30
      ILI9325.docmd("R",@sdbuffer,backgroundaddress,(sizex << 2)) ' get 59*2 *2 bytes from background. Instead of 60 lines with everything multiplied by 2, do 30 lines and multiply by 4
      ILI9325.docmd("W",@sdbuffer,ramaddress,(sizex << 2)) ' save to a temp buffer 
      backgroundaddress += (240 << 2)                          ' next row = 240 pixels *2  *2
      ramaddress += (sizex << 2)                        ' 59*2  *2
    ramaddress := 169984 ' location to store icon
    'PrintStringCR(string("Load"))            ' message on the TV screen
    fat.openfile(@lineoftext1,"R")              ' open an icon image (.icn extension = RGBA)
    repeat sizey
      fat.readdata(@sdbuffer,(sizex << 2))                 ' sizex pixels * 4 bytes per pixel so multiply by 4
      ILI9325.docmd("W",@sdbuffer,ramaddress,(sizex << 2)) ' now move data from hub to external ram
      ramaddress += (sizex << 2)                        ' 4 bytes per pixel so multiply by 4   
    fat.closefile   
    'PrintStringCR(string("Closing file"))
    ILI9325.docmd("F",153600,169984,(sizex*sizey))  ' background,icon,number of pixels


PUB DisplayIcons
    'str.copy(string("clock.icn"),@lineoftext1) ' copy fixed value into a string
    'TransparentIcon(1,1) ' display icon in lineoftext1
    'str.copy(string("cpm.icn"),@lineoftext1) ' copy fixed value into a string  
    'TransparentIcon(2,1) ' display icon in lineoftext1
    str.copy(string("photos.icn"),@lineoftext1) ' copy fixed value into a string
    TransparentIcon(1,1) ' display icon in lineoftext1
    str.copy(string("videos.icn"),@lineoftext1) ' copy fixed value into a string  
    TransparentIcon(2,4) ' display icon in lineoftext1
    str.copy(string("key.icn"),@lineoftext1) ' copy fixed value into a string  
    TransparentIcon(3,0) ' display icon in lineoftext1
    str.copy(string("prop.icn"),@lineoftext1) ' copy fixed value into a string  
    TransparentIcon(0,4) ' display icon in lineoftext1  

PUB TouchStart
  OUTA[Touch_ChipSelect] := 0                           ' enable the touch screen
  SPI.start(3,0)                                        ' delay,state

PUB TouchValue | xval,yval ' returns % values 0-100% (or 255 for no touch)
'  Touch_Clock = 21                                       ' clock
'  Touch_ChipSelect = 23                                  ' chip select needs to be low to enable the touchscreen
'  Touch_DataIn = 22                                      ' data into the chip = an output from the prop
'  Touch_Busy = 20                                        ' busy ? not needed - just returns /cs
'  Touch_DataOut = 20                                     ' data out of the chip = an input to the prop
      SPI.SHIFTOUT(Touch_DataIn, Touch_Clock, 5, 8 , %1101_0000)  ' reads x from  500 to 3700  (off < 500 )
      xval := SPI.SHIFTIN(Touch_DataOut,Touch_Clock,2, 12)
      SPI.SHIFTOUT(Touch_DataIn, Touch_Clock, 5, 8 , %1001_0000)  ' reads y from 400 to 3800 (off > 3800 ) 
      yval := SPI.SHIFTIN(Touch_DataOut,Touch_Clock,2, 12)
      if xval <500                                      ' if less than 500 then not touched, change to 255
        xval := 255                                      ' no touch is 255
      else ' convert 3800-440 to 0-100%
        xval -=  440                                    ' scale 0-3360
        xval /=  32                                     ' scale 0-100
        xval <#= 100                                    ' max 100
        xval #>=0                                       ' min 0
        xval := 100 - xval                              ' so left edge on left
        'printdecimal(xval) 
      if yval > 3800                                    ' if >3800 then not touched, change to 255
        yval := 255                                     ' no touch is 255
      else  ' 3600 is top, 420 is bottom
        yval -= 420                                     ' scale 0-3180
        yval /= 32                                      ' scale 0-100
        yval <#= 100                                    ' max 100
        yval #>=0                                       ' min 0
        yval := 100-yval                                ' so 0 is top left
        'printdecimal(yval)
      result := (yval <<8) | xval                       ' return 00000000_00000000_YYYYYYYY_XXXXXXXX

PUB TouchSwish | xval,oldxval,difference,average ' returns result = +ve or -ve depending on swish direction     ' did finger move left or right?
      oldxval := 255' for startup
      average := 0
      repeat until (average > 30) or (average < -30)
        ' possibly add a decay function here for average
        delay.pause1ms(50)
        xval := TouchValue & 255                        ' decode xval 0-100%
        if xval == 255                                  ' not touched
          oldxval := 255                                ' reset oldxval as well
        else
          if oldxval <>255                                ' discard the first reading as the screen is touched
            difference := xval - oldxval
            'printdecimal(xval)
            'printdecimal(difference)
            average := average + difference             ' 
            'printdecimal(average)
            'crlf
          oldxval := xval                                 ' store for next time
      result := average                                 ' return negative or positive value     


PUB TouchScreen  | yval, xval,x,y
    repeat
      yval := TouchValue >> 8                               ' decode yval 0-100%               
      xval := TouchValue & 255                              ' decode xval 0-100%
      if (xval <> 255)  and (yval <> 255)
        x := xval / 25                                  ' scale 0,1,2,3
        y := yval / 20                                  ' 0,1,2,3,4
        'printdecimal(xval)                              ' display the numbers on TV
        'printdecimal(yval)
        'crlf
        if (x == 0) and (y == 0)
            str.copy(string("twitter.icn"),@lineoftext1) ' copy fixed value into a string
            TransparentIcon(x,y) ' display icon in lineoftext1
        if (x == 1) and (y == 0)
            str.copy(string("calc.icn"),@lineoftext1) ' copy fixed value into a string
            TransparentIcon(x,y) ' display icon in lineoftext1
        if (x == 2) and (y == 0)  
            str.copy(string("calendar.icn"),@lineoftext1) ' copy fixed value into a string
            TransparentIcon(x,y) ' display icon in lineoftext1
        if (x == 3) and (y == 0)
            DrawKeyboard                 ' popup keyboard
            Touchkeyboard
            DrawBackgroundAndIcons  
        if (x == 0) and (y == 1)  
            str.copy(string("cydia.icn"),@lineoftext1) ' copy fixed value into a string
            TransparentIcon(x,y) ' display icon in lineoftext1
        if (x == 1) and (y == 1)
            'str.copy(string("facebook.icn"),@lineoftext1) ' copy fixed value into a string
            'TransparentIcon(x,y) ' display icon in lineoftext1
            PhotoAlbum
            DrawBackgroundAndIcons   
        if (x == 2) and (y == 1)
            str.copy(string("contacts.icn"),@lineoftext1) ' copy fixed value into a string
            TransparentIcon(x,y) ' display icon in lineoftext1
        if (x == 3) and (y == 1)
            str.copy(string("cpm.icn"),@lineoftext1) ' copy fixed value into a string
            TransparentIcon(x,y) ' display icon in lineoftext1
        if (x == 0) and (y == 2)  
            str.copy(string("ifile.icn"),@lineoftext1) ' copy fixed value into a string
            TransparentIcon(x,y) ' display icon in lineoftext1
        if (x == 1) and (y == 2)
            str.copy(string("itunes.icn"),@lineoftext1) ' copy fixed value into a string
            TransparentIcon(x,y) ' display icon in lineoftext1
        if (x == 2) and (y == 2)  
            str.copy(string("phone.icn"),@lineoftext1) ' copy fixed value into a string
            TransparentIcon(x,y) ' display icon in lineoftext1
        if (x == 3) and (y == 2)
            str.copy(string("mail.icn"),@lineoftext1) ' copy fixed value into a string
            TransparentIcon(x,y) ' display icon in lineoftext1
        if (x == 0) and (y == 3)  
            str.copy(string("maps.icn"),@lineoftext1) ' copy fixed value into a string
            TransparentIcon(x,y) ' display icon in lineoftext1
        if (x == 1) and (y == 3)
            str.copy(string("messages.icn"),@lineoftext1) ' copy fixed value into a string
            TransparentIcon(x,y) ' display icon in lineoftext1
        if (x == 2) and (y == 3)
            str.copy(string("music.icn"),@lineoftext1) ' copy fixed value into a string
            TransparentIcon(x,y) ' display icon in lineoftext1
        if (x == 3) and (y == 3)
            str.copy(string("notes.icn"),@lineoftext1) ' copy fixed value into a string
            TransparentIcon(x,y) ' display icon in lineoftext1
        if (x == 0) and (y == 4)  
            Proptool                   ' run the proptool simulator
        if (x == 1) and (y == 4)
            str.copy(string("safari.icn"),@lineoftext1) ' copy fixed value into a string
            TransparentIcon(x,y) ' display icon in lineoftext1
        if (x == 2) and (y == 4)  
            'str.copy(string("settings.icn"),@lineoftext1) ' copy fixed value into a string
            'TransparentIcon(x,y) ' display icon in lineoftext1
            Movie ' play movie if hit the movie icon
            DrawBackgroundAndIcons
        if (x == 3) and (y == 4)
            str.copy(string("stocks.icn"),@lineoftext1) ' copy fixed value into a string
            TransparentIcon(x,y) ' display icon in lineoftext1

        delay.pause1ms(250)' only display once so delay half a second to "debounce"

PUB Proptool | filesize,pointer,c
     str.copy(string("demo.txt"),@lineoftext1)          ' new faster font file - one file, designed to be read as fast as possible
     filesize := Loadfile                                           ' load in the text to external ram
     ILISetcursor(0,0)                                  ' sets curx and cury to zero
     NewFont(string("ParBY16.ifn"))                     ' brown on yellow
     pointer := RamText
     repeat filesize
       c := ILI9325.readrambyte(pointer)
       if c == 13
          ILIcrlf                                       ' new line, curx to zero and cury down by one font height

       case pointer - Ramtext
         $28:  NewFont(string("ParbBY16.ifn"))          ' bold black on yellow
         $8B:  NewFont(string("ParGY16.ifn"))           ' green on yellow
         $96:  NewFont(string("ParbBR16.ifn"))          ' bold black on red
         $9B:  NewFont(string("ParBR16.ifn"))           ' black on red
         $12E: NewFont(string("ParbBO16.ifn"))          ' bold black on orange
         $133: NewFont(string("ParBO16.ifn"))           ' black on orange
         $14D: NewFont(string("ParOO16.ifn"))           ' orange on orange
         $16E: NewFont(string("ParBO16.ifn"))           ' black on orange
         $186: NewFont(string("ParOO16.ifn"))           ' orange on orange
         $1A7: NewFont(string("ParBO16.ifn"))           ' black on orange
         $1BF: NewFont(string("ParOO16.ifn"))           ' orange on orange
         '$1DD: NewFont(string("ParBO16.ifn"))          ' black on orange
         $1E2: NewFont(string("ParbBC16.ifn"))          ' bold black on cyan
         $1E5: NewFont(string("ParBC16.ifn"))           ' black on cyan
         $2A0: NewFont(string("ParRC16.ifn"))           ' red on cyan     
         $307: NewFont(string("ParCC16.ifn"))           ' cyan on cyan for comments
         $313: NewFont(string("ParBC16.ifn"))           ' black on cyan
         $3C2: NewFont(string("ParbBC16.ifn"))          ' bold black on cyan
         
       if (c=>32) and (c=< 126)
         curx := ILIChar(c,curx,cury)                   ' only print if a valid ascii character
       pointer ++
     
PUB NewFont(strpointer)
    str.copy(strpointer,@lineoftext1)          ' new faster font file - one file, designed to be read as fast as possible
    ILIloadfont
    ClearFontBackground(curx,cury,239,cury+FontHeight)             ' clear an area the same color as the font background  
     

PUB LoadFile | filesize,ramaddress ' reads a file in lineoftext1 into the external ram
    fat.openfile(@lineoftext1,"R")            ' open a file
    filesize := fat.filesize
    
    ramaddress := RamText                               ' store text here
    repeat (filesize >>8 ) + 1                          ' so works with 256 byte blocks
      fat.readdata(@sdbuffer,256)                       ' get 256 bytes from sd card
      ILI9325.docmd("A",@sdbuffer,ramaddress,256)       ' A must start on 0,256,512,768 etc move data from hub to external ram 256 bytes at a time    
      ramaddress := ramaddress + 256                    ' add 256
    fat.closefile   
    return filesize
 
PUB PrintStringCR(pstring)' sends pstring to the vga and serial port
     PrintString(pstring)                               ' print the string  
     crlf

PUB PrintString(pstring)' sends pstring to the vga and serial port with no CRLF
    if (_TV_Enabled)
      text.str(pstring)
'    if (_VGA_Enabled)  
'      vt100.str(pstring)                                 ' send to vga screen
'    if (_Serial_Enabled)   
'    sio.str(0,pstring)                                 ' send to com port
'    if (_lcd_Enabled)
'       lcd.str(pstring)                                   ' send to 20x4 lCD display

PUB PrintChar(c) ' sends c to vga and serial port
    if (_TV_Enabled)
      text.out(c)
'    if (_VGA_Enabled)
'      vt100.out(c)                                        ' send to vga
'    if (_Serial_Enabled)
'      sio.tx(0,c)                                         ' send to com port
'    if (_lcd_Enabled)
'      LCD.out(c)                                          ' send to LCD display      
PUB CRLF
    PrintChar(13)                                        ' send to vga
    PrintChar(10)                                        ' send to vga

PUB PrintDecimal(number)                                ' print the decimal value, useful for debugging
     PrintString(str.integerToDecimal(number, 10))

PUB PrintHex(number)
    Printstring(str.integertohexadecimal(number,8)) ' print the hex long value on the TV

DAT
sdbuffer        byte    $0[512]  ' 512 byte buffer for sd card interface
buffer2         byte    $0[512]  ' 512 general purpose hub buffer